//
// Created by peter on 11/29/19.
//

#include "fmcReadmciniEvent.h"

fmcReadmciniEvent::fmcReadmciniEvent() : fChain(0),
fEvent(0),
fIniState(0)
{
}

fmcReadmciniEvent::~fmcReadmciniEvent()
{
    if (fChain && fEvent && fIniState)
      delete fChain->GetCurrentFile();
    delete fEvent;
    delete fIniState;
}

Long64_t fmcReadmciniEvent::LoadTree(Long64_t entry)
{
    // Set the environment to read one entry
    if (!fChain)
        return -5;
    Long64_t centry = fChain->LoadTree(entry);
    if (centry < 0)
        return centry;
    if (fChain->GetTreeNumber() != fCurrent)
    {
        fCurrent = fChain->GetTreeNumber();
        Notify();
    }
    return centry;
}

void fmcReadmciniEvent::Init(class TTree * tree)
{
    // Set branch addresses and branch pointers
    if (!tree)
        return;
    fChain = tree;
    //fCurrent = -1;
    //fChain->SetMakeClass(1);

    tree->SetBranchAddress("iniState", &fIniState);
    tree->SetBranchAddress("event", &fEvent);
}

Bool_t fmcReadmciniEvent::Notify()
{
    // The Notify() function is called when a new file is opened. This
    // can be either for a new TTree in a TChain or when when a new TTree
    // is started when using PROOF. It is normally not necessary to make changes
    // to the generated code, but the routine can be extended by the
    // user if needed. The return value is currently not used.

    return kTRUE;
}

Long64_t fmcReadmciniEvent::GetEntries()
{
    return fChain->GetEntries();
}

Long64_t fmcReadmciniEvent::GetEntriesFast()
{
    return fChain->GetEntriesFast();
}

fmcInt fmcReadmciniEvent::GetNparticle()
{
    return fEvent->GetNpa();
}

fmcEventHeader *fmcReadmciniEvent::ReadEventHeader(fmcLong iev)
{
    //Long64_t ientry = LoadTree(iev);
    //if (ientry < 0)
    //    return nullptr;

    fmcEventHeader *header = new fmcEventHeader();
    fChain->GetEntry(iev);

    header->SetB(fEvent->GetB());
    header->SetNparts(GetNparticle());
    header->SetPhi(fEvent->GetPhi());

    return header;
}

fmcParticle *fmcReadmciniEvent::ReadParticle(fmcInt ipart)
{
    if (ipart<0 || ipart >= GetNparticle()) return nullptr;
    fmcParticle *particle = new fmcParticle();
    UParticle *uparticle = fEvent->GetParticle(ipart);
    if (!uparticle) return nullptr;

    particle->SetPdg(uparticle->GetPdg());
    particle->SetCharge(1./3.*TDatabasePDG::Instance()->GetParticle(uparticle->GetPdg())->Charge());
    particle->SetPxPyPzE(uparticle->Px(), uparticle->Py(),
                         uparticle->Pz(), uparticle->E());

    return particle;
}
