#ifndef FMC_READER_MCPICO_H
#define FMC_READER_MCPICO_H

#include <TROOT.h>
#include <TChain.h>
#include <TFile.h>

#include <algorithm>

#include "fmcEvent.h"

#define MAX_TRACKS 20000

class fmcReadmcpicoEvent
{
private:
    TTree          *fChain;   //!pointer to the analyzed TTree or TChain
    Int_t           fCurrent; //!current Tree number in a TChain

    // Fixed size dimensions of array or collections stored in the TTree if any.

    // Declaration of leaf types
    Float_t         bimp;
    Float_t         phi2;
    Float_t         phi3;
    Float_t         ecc2;
    Float_t         ecc3;
    Int_t           npart;
    Int_t           nh;
    Float_t         momx[MAX_TRACKS];   //[nh]
    Float_t         momy[MAX_TRACKS];   //[nh]
    Float_t         momz[MAX_TRACKS];   //[nh]
    Float_t         ene[MAX_TRACKS];   //[nh]
    Int_t           hid[MAX_TRACKS];   //[nh]
    Int_t           pdg[MAX_TRACKS];   //[nh]
    Short_t         charge[MAX_TRACKS];   //[nh]

    // List of branches
    TBranch        *b_bimp;   //!
    TBranch        *b_phi2;   //!
    TBranch        *b_phi3;   //!
    TBranch        *b_ecc2;   //!
    TBranch        *b_ecc3;   //!
    TBranch        *b_npart;   //!
    TBranch        *b_nh;   //!
    TBranch        *b_momx;   //!
    TBranch        *b_momy;   //!
    TBranch        *b_momz;   //!
    TBranch        *b_ene;   //!
    TBranch        *b_hid;   //!
    TBranch        *b_pdg;   //!
    TBranch        *b_charge;   //!

public:
    fmcReadmcpicoEvent();
    virtual ~fmcReadmcpicoEvent();

    virtual Int_t    Cut(Long64_t entry);
    virtual Int_t    GetEntry(Long64_t entry);
    virtual Long64_t LoadTree(Long64_t entry);
    virtual void     Init(TTree *tree);
    virtual void     Loop();
    virtual Bool_t   Notify();
    virtual void     Show(Long64_t entry = -1);
    virtual Long64_t GetEntries();
    virtual Long64_t GetEntriesFast();
    virtual fmcInt   GetNparticle();

    virtual fmcEvent *ReadEvent(fmcLong iev);
    virtual fmcEventHeader *ReadEventHeader(fmcLong iev);
    virtual fmcParticle *ReadParticle(fmcInt ipart);
};

#endif