#include <vector>
#include <TH1I.h>
#include <TMath.h>
#include <Math/SpecFuncMathMore.h>

namespace fmc
{
// Struct for centrality determination
struct Lim
{
    std::vector<fmcUShort> L;
    std::vector<fmcUShort> H;
};
// (c) Viktar Kireyeu

// Resolution calculation
//S----------------------------------------------------------------
double GetRes(double _chi, double _harm)
{
    double con = TMath::Sqrt(TMath::Pi() / 2) / 2;
    double arg = _chi * _chi / 4.;
    double order1 = (_harm - 1) / 2.;
    double order2 = (_harm + 1) / 2.;
    double res = con * _chi * exp(-arg) * (ROOT::Math::cyl_bessel_i(order1, arg) + ROOT::Math::cyl_bessel_i(order2, arg));
    return res;
}
//E----------------------------------------------------------------

// Chi2 calculation
//S----------------------------------------------------------------
double GetChi(double _res, double _harm, int accuracy)
{
    double chi = 2.0;
    double delta = 1.0;
    for (int i = 0; i < accuracy; i++)
    {
        if (GetRes(chi, _harm) < _res)
            chi = chi + delta;
        else
            chi = chi - delta;
        delta = delta / 2.;
    }
    return chi;
}
//E----------------------------------------------------------------

// Calculate centrality bins borders
//S----------------------------------------------------------------
Lim CentralityBorders(TH1I *const &nCh, double efficiency = 1.)
{
    int HistLim = nCh->GetNbinsX();
    double Integral = nCh->Integral(1, HistLim);
    double tmpIntegral;
    int RightLim;
    Lim lim;
    // double efficiency = 0.93;
    double BinWidth = 10.;
    double Nbins = 100. / BinWidth;
    double fraction = (Integral / 100.) * efficiency * BinWidth;

    printf("Integral: %8.0f\n", Integral);
    printf("Centrality bins width: %2.1f, number of bins: %2.0f\n", BinWidth, Nbins);
    printf("Trigger efficiency: %1.2f\n", efficiency);
    printf("Fraction of events in each centrality bin: %8.2f\n", fraction);

    RightLim = HistLim;
    int j = 0;
    for (int i = HistLim; i > 0; --i)
    {
        tmpIntegral = nCh->Integral(i, RightLim);
        if (tmpIntegral >= fraction)
        {
            printf("[%d,%d]: %8.2f\n", i, RightLim, tmpIntegral);
            lim.H.push_back(RightLim);
            lim.L.push_back(i);
            ++j;
            RightLim = i - 1;
        }
    }
    return lim;
}
//E----------------------------------------------------------------
// (c) Viktar Kireyeu

// Get Centrality bin from refMult
fmcInt GetCentrality(Lim lim, fmcInt mult)
{
    fmcInt CentBin = -1;

    // --- Centrality bins selection
    for (int i = 0; i < lim.L.size(); ++i)
    {
        if (mult >= lim.L.at(i) && mult <= lim.H.at(i))
        {
            CentBin = i;
            break;
        }
        else
            CentBin = 10;
    }

    CentBin *= 10;
    return CentBin;
}
} // namespace fmc