#include <iostream>
#include <vector>

#include <TStopwatch.h>
#include <TString.h>
#include <TFile.h>
#include <TH1F.h>
#include <TProfile.h>

#include "fmcTypes.h"
#include "fmcConstants.h"
#include "fmcFunctions.cxx"

int main(int argc, char **argv)
{
    TString inFileName, outFileName;
    if (argc < 5)
    {
        std::cerr << "./calcRes -i input -o output" << std::endl;
        return 10;
    }
    for (int i = 1; i < argc; i++)
    {
        if (std::string(argv[i]) != "-i" &&
            std::string(argv[i]) != "-o")
        {
            std::cerr << "\ncalcRes: Unknown parameter " << i
                      << ": " << argv[i] << std::endl;
            return 11;
        }
        else
        {
            if (std::string(argv[i]) == "-i" && i != argc - 1)
            {
                inFileName = argv[++i];
            }
            if (std::string(argv[i]) == "-i" && i == argc - 1)
            {
                std::cerr << "\ncalcRes: Input file name was not specified!" << std::endl;
                return 20;
            }
            if (std::string(argv[i]) == "-o" && i != argc - 1)
            {
                outFileName = argv[++i];
            }
            if (std::string(argv[i]) == "-o" && i == argc - 1)
            {
                std::cerr << "\ncalcRes: Output file name was not specified!" << std::endl;
                return 21;
            }
        }
    }

    TFile *fi = new TFile(inFileName.Data(), "read");
    if (fi->IsZombie())
        return 1;
    TProfile *pres;
    std::vector<fmcDouble> vRes2[fmc::nTpcEtaGaps], vEres2[fmc::nTpcEtaGaps];
    std::vector<fmcDouble> vRes2FHCal, vEres2FHCal;
    for (int i = 0; i < fmc::nTpcEtaGaps; i++)
    {
        pres = (TProfile *)fi->Get(Form("Res2/presTPC_gap%i", i));
        if (!pres)
        {
            std::cerr << "Error: histogram " << Form("presTPC_gap%i", i)
                      << " was not found in file!" << std::endl;
            return 2;
        }
        for (int j = 1; j <= pres->GetNbinsX(); j++)
        {
            vRes2[i].push_back(pres->GetBinContent(j));
            vEres2[i].push_back(pres->GetBinError(j));
        }
        delete pres;
    }
    pres = (TProfile *)fi->Get(Form("Res2/presFHCal"));
    if (!pres)
    {
        std::cerr << "Error: histogram " << Form("presFHCal")
                  << " was not found in file!" << std::endl;
        return 2;
    }
    for (int j = 1; j <= pres->GetNbinsX(); j++)
    {
        vRes2FHCal.push_back(pres->GetBinContent(j));
        vEres2FHCal.push_back(pres->GetBinError(j));
    }
    delete pres;
    fi->Close();

    // Manage output profiles
    TH1F *pout[fmc::nTpcEtaGaps];
    TH1F *pFHCal = new TH1F("p_resFHCal","p_resFHCal", 10, 0, 100);
    for (int i = 0; i < fmc::nTpcEtaGaps; i++)
    {
        pout[i] = new TH1F(Form("p_resTPC_gap%i", i),
                           Form("p_resTPC_gap%i", i), 10, 0, 100);
    }

    fmcDouble res2, res, Eres2, Eres;
    for (int i = 0; i < fmc::nTpcEtaGaps; i++)
    {
        for (int j = 0; j < vRes2[i].size(); j++)
        {
            res2 = vRes2[i].at(j);
            Eres2 = vEres2[i].at(j);
            if (res2 <= 0)
                continue;

            res = TMath::Sqrt(res2);
            Eres = 0.5 * Eres2 / TMath::Sqrt(res2);
            pout[i]->SetBinContent(j+1, res);
            pout[i]->SetBinError(j+1, Eres);
        }
    }
    fmcDouble chi, chiFull, resFull;
    for (int i = 0; i < vRes2FHCal.size(); i++)
    {
      res2 = vRes2FHCal.at(i);
      Eres2 = vEres2FHCal.at(i);
      if (res2 <= 0) continue;

      res = TMath::Sqrt(res2);
      chi = fmc::GetChi(res, 2, 50);
      chiFull = TMath::Sqrt(2)*chi;
      resFull = fmc::GetRes(chi, 2);
      pFHCal->SetBinContent(i+1, resFull);
    }

    // Write output
    TFile *fo = new TFile(outFileName.Data(), "recreate");
    fo->cd();
    for (int i = 0; i < fmc::nTpcEtaGaps; i++)
    {
        pout[i]->Write();
    }
    pFHCal->Write();
    fo->Close();

    return 0;
}
