#include <iostream>
#include <fstream>

#include <TStopwatch.h>
#include <TString.h>
#include <TChain.h>

#include "fmcTypes.h"
#include "fmcReader.h"
#include "fmcEvent.h"
#include "fmcEventHeader.h"
#include "fmcParticle.h"

int main(int argc, char **argv)
{
    TString inFilelistName;
    if (argc < 3)
    {
        std::cerr << "./simpleLoop -i filelist" << std::endl;
        return 10;
    }
    for (int i = 1; i < argc; i++)
    {
        if (std::string(argv[i]) != "-i")
        {
            std::cerr << "\nsimpleLoop: Unknown parameter " << i
                      << ": " << argv[i] << std::endl;
            return 11;
        }
        else
        {
            if (std::string(argv[i]) == "-i" && i != argc - 1)
            {
                inFilelistName = argv[++i];
            }
            if (std::string(argv[i]) == "-i" && i == argc - 1)
            {
                std::cerr << "\nsimpleLoop: Input file name was not specified!" << std::endl;
                return 20;
            }
        }
    }

    TChain *chain = new TChain("mctree");
    std::fstream file(inFilelistName.Data());
    std::string str;
    int nfiles = 0;

    while (std::getline(file, str))
    {
        if (chain->Add(str.c_str()))
            nfiles++;
    }

    std::cout << "Processing simpleLoop..." << std::endl;
    std::cout << "Found " << nfiles << " files in the filelist." << std::endl;
    fmcReader *reader = new fmcReader();
    reader->Init(chain);
    // reader->Loop();

    fmcLong Nevents = reader->GetEntries();
    std::cout << "SimpleLoop: found " << Nevents << " events." << std::endl;

    fmcEventHeader *header = nullptr;
    fmcParticle *particle = nullptr;
    fmcInt Nparts;

    TStopwatch timer;
    timer.Start();

    for (fmcLong iEv = 0; iEv < Nevents; iEv++)
    {
        if (iEv % 1000 == 0)
            std::cout << "SimpleLoop: Event [" << iEv << "/"
                      << Nevents << "]" << std::endl;

        header = reader->ReadEventHeader(iEv);


        // main process with event header


        Nparts = reader->GetNparticle();
        for (int iPart = 0; iPart < Nparts; iPart++)
        {
            particle = reader->ReadParticle(iPart);
            if (!particle)
                continue;
            if (particle->GetCharge() == 0)
            {
                delete particle;
                continue;
            }


            // main process with particle loop


            delete particle;
        }

        delete header;
    }

    timer.Stop();
    timer.Print();

    return 0;
}
